local Addon, TA = ...

-- Generic Locals
local on = true
local off = false
local PerSec = 8 -- run the Queue scan X times a second
local AnnounceDupeTime = 1 -- Duplicate announce filter time (if same spell happens within x seconds by same person it won't announce it)

-- Addon Settings
TauntAware_Settings = {}
local TAS = nil 
TauntAware_Default_Settings = {
	Enable = on,
	Full = on,
	Links = on,
	Scan = on,
	Tattle = off,
}


-- Bill's Utils
local SJprint = BillsUtils.SJprint
local addOptionMt = BillsUtils.addOptionMt
local StatColor = BillsUtils.StatColor
local OptSaveTF = BillsUtils.OptSaveTF
local CCC = BillsUtils.CCC

BillsUtils.Locals[#BillsUtils.Locals +1] = function ()
	SJprint = BillsUtils.SJprint
	addOptionMt = BillsUtils.addOptionMt
	StatColor = BillsUtils.StatColor
	OptSaveTF = BillsUtils.OptSaveTF
	CCC = BillsUtils.CCC
end

-- Local text colors
local yel = "|cFFFFFF00"
local wht = "|cFFFFFFFF"
local blu = "|cFF00FFFF"
local red = "|cFFFF0000"
local grn = "|cFF00FF00"
local res = "|r"

local SpellList = {}
-- SpellId, Detect Type
-- 1 = SPELL_CAST_SUCCESS
-- 2 = SPELL_AURA_APPLIED

SpellList = {
	--DRUID
	--SPELL_CAST_SUCCESS
	[6795] = true,	-- Growl
	[5209] = true,	-- Challenging Roar

	--HUNTER
	--SPELL_CAST_SUCCESS
	[34477] = true,	-- Misdirection
	[20736] = true,	-- Distracting Shot

	--MAGE

	--PALADIN
	--SPELL_CAST_SUCCESS
	[31789] = true,	-- Righteous Defense
	[62124] = true,	-- Hand of Reckoning
	[25780] = true,	-- Righteous Fury

	--PRIEST
	
	--ROGUE
	--SPELL_CAST_SUCCESS
	[57934] = true,	-- Tricks of the Trade

	--SHAMAN
	--SPELL_CAST_SUCCESS
	[8056] = true,	-- Frost Shock
	--"SPELL_AURA_APPLIED"
	[73684] = true,	-- Unleash Earth

	--WARLOCK

	--WARRIOR
	--SPELL_CAST_SUCCESS
	[355] = true,	-- Taunt
	[1161] = true,	-- Fury
	[57755] = true,	-- Heroic Throw
	[71] = true,	-- Defensive Stance

	--DEATHKNIGHT
	--SPELL_CAST_SUCCESS
	[48263] = true,	-- Blood Presence
	[49576] = true,	-- Death Grip
	[56222] =  true,	-- Dark Command

	--PETS
	-- Warlock Pets
	[17735] = true,	-- Suffering
	[3716] = true,	-- Torment
	-- Hunter Pets
	[2649] = true,	-- Growl
	[53477] = true,	-- Taunt
	[24394] = true,	-- Intimidation
	[63900] = true,	-- Thunderstomp
}

local scanTool = CreateFrame( "GameTooltip", "ScanTooltip", nil, "GameTooltipTemplate" )
scanTool:SetOwner( WorldFrame, "ANCHOR_NONE" )
local scanText = _G["ScanTooltipTextLeft2"] -- This is the line with <[Player]'s Pet>

function getPetOwner(petName)
   scanTool:ClearLines()
   scanTool:SetUnit(petName)
   local ownerText = scanText:GetText()
   if not ownerText then return nil end
   local owner, _ = string.split("'",ownerText)   
   return owner -- This is the pet's owner
end
local Queue = {}

TA.Frame = CreateFrame("Frame")
TA.Frame:RegisterEvent("ADDON_LOADED")
TA.Frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
TA.Frame:RegisterEvent("PARTY_MEMBERS_CHANGED")
TA.Frame:RegisterEvent("RAID_ROSTER_UPDATE")

TA.Frame:SetScript("OnEvent",function(self, event, ...)
	local arg = ...
	if event == "ADDON_LOADED" and arg == Addon then
		SJprint(yel, "Taunt Aware v",red, GetAddOnMetadata(Addon, "Version"), yel, "loaded", res)
		addOptionMt( TauntAware_Settings, TauntAware_Default_Settings )
		TAS = TauntAware_Settings
		return self, event, ...
	end
	if not(TAS.Enable) then return end
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		Queue[#Queue +1] = { ... }
		return
	end
	if event == "PARTY_MEMBERS_CHANGED" or event == "RAID_ROSTER_UPDATE" then
		-- raid / party scan
		return
	end	
	return
end)

local e = 0
local last = {}
TA.Frame:SetScript("OnUpdate", function(self, elapsed)
	e = e + elapsed
	if e < 1/PerSec then
		return
	end
	e = 0
	
	if not(TAS.Enable) then return end
	while #Queue > 0 do
		local timestamp, eventType, hidecaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID, destName, destFlags, destFlags2, spellId, spellName = unpack(Queue[1])
		local PlayerRole = UnitGroupRolesAssigned("player")
		if UnitExists(sourceName) and SpellList[spellId] and ( eventType == "SPELL_CAST_SUCCESS" or eventType == "SPELL_AURA_APPLIED") then
			local B = tonumber(sourceGUID:sub(5,5), 16);
			local maskedB = B % 8; -- x % 8 has the same effect as x & 0x7 on numbers <= 0xf
			-- local knownTypes = {[0]="player", [3]="NPC", [4]="pet", [5]="vehicle"};
	
			local Self = UnitIsUnit(sourceName, "player")
			local SelfPet = UnitIsUnit(sourceName, "pet")
			local PC = maskedB == 0 
			local Pet = maskedB == 4
			
			local Announce = false
			if TAS.Full or PlayerRole == "TANK" then
				Announce = true
			elseif PlayerRole == "DAMAGER" and (Self or SelfPet) then
				SJprint(red, "Are You supposed to be taunting?")
				Announce = true
			elseif PlayerRole == "HEALER" then
				--whatever for healers
			elseif PlayerRole == "NONE" then
				Announce = true
			end
			
			--Duplicate announce check
			if Announce and last.sourceName == sourceName and last.spellId == spellId and (timestamp - last.timestamp < AnnounceDupeTime ) then
				Announce = false
			end
				
			if Announce then
				last.sourceName = sourceName
				last.spellId = spellId
				last.timestamp = timestamp
				
				local _, sourceClass = UnitClass(sourceName)
				local name = ""
	
				if Pet and TAS.Links then
					spellName = "|cff71d5ff|Hspell:"..spellId.."|h["..spellName.."]|h|r" -- Create Spell Link
					local PetOwner = getPetOwner(sourceName)
					local _, PetOwnerClass = UnitClass( PetOwner )
					name = ("%s|Hplayer:%s|h[%s]|h %s%s "):format( CCC(PetOwnerClass), PetOwner, PetOwner, CCC(sourceClass), sourceName )
				elseif Pet then
					local PetOwner = getPetOwner(sourceName)
					local _, PetOwnerClass = UnitClass( PetOwner )
					name = ("%s%s %s%s"):format( CCC(PetOwnerClass), PetOwner, sourceClass, sourceName )
				elseif TAS.Links then
					spellName = "|cff71d5ff|Hspell:"..spellId.."|h["..spellName.."]|h|r" -- Create Spell Link
					name = ("%s|Hplayer:%s|h[%s]|h"):format(CCC(sourceClass), sourceName, sourceName)
				else
					name = ("%s%s"):format(sourceClass, sourceName) 
				end
		
				local msg = ""
				
		
				if destName then
					msg = string.format("|cFFFFFF00<Taunt Aware> %s|cFFFFFFFF casts %s on |cFFFF0000%s|r", name, spellName, destName )
				else
					msg = string.format("|cFFFFFF00<Taunt Aware> %s|cFFFFFFFF casts %s|r", name, spellName )
				end
		
				print( msg )
			end
		end
		table.remove(Queue, 1)
	end
end)


TA.CmdLine = function( msg )
	local cmd,arg = string.split(" ", msg)
	cmd = cmd:lower()
	
--	if cmd =="config" then
--		InterfaceOptionsFrame_OpenToCategory( TA.panel )
--		return
--	end
	
	if not(TAS.Enable) and cmd ~= "enable" then
		print(" ")
		SJprint( yel, "Taunt Aware", res)
		SJprint( yel, "  This addon is ", red, "DISABLED", res)
		SJprint( yel, "  Type: ", blu, "/achsnd enable on", res)
		SJprint( yel, "  to enable.", res)
		return
	end
	
	--On/Off True/False values
	local Command = {"Enable", "Full", "Links", "Scan", "Tattle",}
	
	for x = 1, #Command do
		if cmd == string.lower(Command[x]) then
			OptSaveTF( TAS, Command[x], arg)
			SJprint(  yel, "Taunt Aware:", Command[x],"is", StatColor( TAS[Command[x]] ), res)
			return
		end
	end

	if cmd == "" then
		print(" ")
		SJprint( yel, "________________________________________", res)
		SJprint( yel, "Taunt Aware", res)
		SJprint( blu, "     Enable", wht, "is", StatColor(TAS.Enable), res)
		SJprint( blu, "     Full", wht, "alerts are", StatColor(TAS.Full), res)
		SJprint( blu, "     Links ", wht, "for abilities are", StatColor(TAS.Links), res)
		SJprint( blu, "     Scan ", wht, "on party change is", StatColor(TAS.Scan), res)
		SJprint( blu, "     Tattle ", wht, "on non tank taunts is", StatColor(TAS.Tattle), res)
		SJprint( yel, "Available commands are listed above in", blu, "BLUE.", res)
--		SJprint( yel, "Type", blu, "/ta config", yel, "to open the GUI config screen.", res)
--		SJprint( yel, "Type", blu, "/ta cmdlist", yel, "for a description of commands.", res)
		SJprint( yel, "________________________________________", res)
		return
	end

	print(" ")
	SJprint( yel, "  Taunt Aware", res)
	SJprint( yel, "  Error: ", red, cmd, yel, " is not a valid command", res)
	SJprint( yel, "  Type: ", blu, "/ta", yel, "for the addon status and command list", res)
--	SJprint( yel, "  Type: ", blu, "/ta cmdlist", yel, "for command list help", res)
	return
end


SLASH_TA1 = "/ta"
SlashCmdList["TA"] = function(msg)
	TA.CmdLine(msg)
end
